# Copyright (c) 2000 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from narval.lib import *
import copy, os
import urllib, urlparse, string

BUG_NS=''

MOD_XML = """<module name='Buggy'>"""

def compute_bug_id_f(args):
    doc = args.ownerDocument
    outputs = doc.createElementNS(BUG_NS,'outputs')

    path = expand_vars(select_one(args,'db-path/text()').data)
    bug_tree = read_domtree_from_file(path).documentElement
    bug_list = select_many(bug_tree,'//bug')
    bug_id = 1
    if bug_list:
        last_bug = bug_list[len(bug_list)-1]
        last_bug_id = int(last_bug.getAttributeNS(BUG_NS,'id'))
        bug_id = last_bug_id + 1
    bug = select_one(args,'bug')
    bug.setAttributeNS(BUG_NS,'id',str(bug_id))
    outputs.appendChild(bug)
    return outputs


MOD_XML = MOD_XML + \
          """<action name='compute_bug_id' func='compute_bug_id_f'>
          <input><match>db-path</match></input>
          <input id='former-bug' outdates='yes'><match>bug[@state='not-assigned']</match></input>
          <output outdates='former-bug'><match>bug[@state='not-assigned']</match></output>
          </action>
          """

def compute_workers_dict_f(args):
    doc = args.ownerDocument
    outputs = doc.createElementNS(BUG_NS,'outputs')

    bug_list = select_many(args,'.//bug[@state="assigned"]')
    workers = doc.createElementNS(BUG_NS,'workers')
    outputs.appendChild(workers)
    workers_dict = {}
    for bug in bug_list:
        w = select_one(bug,'worker')
        w_id = w.getAttributeNS(BUG_NS,'id')
        date = w.getAttributeNS(BUG_NS,'date')
        bug_id = bug.getAttributeNS(BUG_NS,'id')
        label = select_one(bug,'label/text()').data
        if workers_dict.has_key(w_id):
            workers_dict[w_id].append((bug_id,label,date))
        else:
            workers_dict[w_id] = [(bug_id,label,date)]

    for w_id, list in workers_dict.items():
        worker = doc.createElementNS(BUG_NS,'worker')
        worker.setAttributeNS(BUG_NS,'id',w_id)
        for bug_id,label,date in list:
            bug = doc.createElementNS(BUG_NS,'bug-element')
            bug.setAttributeNS(BUG_NS,'id',bug_id)
            bug.setAttributeNS(BUG_NS,'label',label)
            bug.setAttributeNS(BUG_NS,'date',date)
            worker.appendChild(bug)
        workers.appendChild(worker)
    return outputs

MOD_XML = MOD_XML + \
          """<action name='compute_workers_dict' func='compute_workers_dict_f'>
          <input><match>bugnicar-database</match></input>
          <output><match>workers</match></output>
          </action>
          """


# Test #########################################################################

MOD_XML = MOD_XML + "</module>"

if __name__ == "__main__" :
    from xml.dom.ext.reader import Sax2
    print MOD_XML
    doc=Sax2.FromXml(MOD_XML)
    print doc.documentElement
